/*  SQuaLe
 *
 *  Copyright (C) 2004 Julien Moutte <julien@moutte.net>
 *
 *  squale.c : Source code for the SQuaLe php client module.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
#include "php_squale.h"

#include <squaleclient.h>

#define le_squaleresult_name  "SqualeResult"

/* True global resources - no need for thread safety here */
static int le_squale;
static int le_squaleresult;

/* {{{ squale_functions[]
 *
 * Every user visible function must have an entry in squale_functions[].
 */
function_entry squale_functions[] = {
  PHP_FE (squale_run_query, NULL)
  PHP_FE (squale_result_get_type, NULL)
  PHP_FE (squale_result_get_processing_time, NULL)
  PHP_FE (squale_result_get_assignation_time, NULL)
  PHP_FE (squale_result_get_error, NULL)
  PHP_FE (squale_result_get_warning, NULL)
  PHP_FE (squale_result_get_affected_rows, NULL)
  PHP_FE (squale_result_get_nb_rows, NULL)
  PHP_FE (squale_result_get_nb_cols, NULL)
  PHP_FE (squale_result_get_column_names, NULL)
  PHP_FE (squale_result_get_value, NULL)
  PHP_FE (squale_result_get_row, NULL)
  PHP_FE (squale_result_get_resultset, NULL)
  {NULL, NULL, NULL} /* Must be the last line in squale_functions[] */
};
/* }}} */

/* {{{ squale_module_entry
 */
zend_module_entry squale_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
  STANDARD_MODULE_HEADER,
#endif
  "squale",
  squale_functions,
  PHP_MINIT (squale),
  PHP_MSHUTDOWN (squale),
  NULL,
  NULL,
  PHP_MINFO (squale),
#if ZEND_MODULE_API_NO >= 20010901
  "0.0.4", /* Replace with version number for your extension */
#endif
  STANDARD_MODULE_PROPERTIES
};
/* }}} */

#ifdef COMPILE_DL_SQUALE
ZEND_GET_MODULE (squale)
#endif

void
squale_result_destruction_handler (zend_rsrc_list_entry *rsrc TSRMLS_DC) {
  SqualeResult *result = (SqualeResult *) rsrc->ptr;
  
  if (result) {
    squale_free_result (result);
    rsrc->ptr = NULL;
  }
}

/* {{{ PHP_MINIT_FUNCTION
 */
PHP_MINIT_FUNCTION (squale)
{
  le_squaleresult = zend_register_list_destructors_ex (
                      squale_result_destruction_handler, NULL,
                      le_squaleresult_name, module_number);
  
  return SUCCESS;
}
/* }}} */

/* {{{ PHP_MSHUTDOWN_FUNCTION
 */
PHP_MSHUTDOWN_FUNCTION (squale)
{
  return SUCCESS;
}
/* }}} */

/* {{{ PHP_MINFO_FUNCTION
 */
PHP_MINFO_FUNCTION (squale)
{
  php_info_print_table_start ();
  php_info_print_table_header (2, "squale support", "enabled");
  php_info_print_table_end ();
}
/* }}} */

/* {{{ proto resource squale_run_query (string connection_name,
                                        string query, string socket_name,
                                        long timeout)
   This function runs a query through SQuaLe on a specific connection name.
   It uses the unix socket specified in the socket_name parameter and waits
   for a maximum number of seconds defined by the optional timeout parameter. */
PHP_FUNCTION (squale_run_query)
{
  char *connection_name = NULL, *query = NULL, *socket_name = NULL;
  int connection_name_len, query_length, socket_name_length;
  long timeout = 0;
  SqualeResult *result = NULL;
  zval *resource;
  
  if (zend_parse_parameters (ZEND_NUM_ARGS () TSRMLS_CC, "sss|l",
                             &connection_name, &connection_name_len,
                             &query, &query_length,
                             &socket_name, &socket_name_length,
                             &timeout) == FAILURE) {
    return;
  }
  
  result = squale_run_query (connection_name, query, socket_name, timeout);
  
  if (result) {
    int rsrc_id;
    MAKE_STD_ZVAL (resource);
    rsrc_id = ZEND_REGISTER_RESOURCE (resource, result, le_squaleresult);
    RETURN_RESOURCE (rsrc_id);
  }
  else {
    zend_error (E_WARNING,
                "Invalid return value from squale_run_query C function call.");
    RETURN_NULL ();
  }
}
/* }}} */

/* {{{ proto long squale_result_get_type (resource result)
   This function gives you the type of result that squale_run_query generated.
   This will be 0 for an error, 1 for some affected rows, and 2 for a
   resultset. */
PHP_FUNCTION (squale_result_get_type)
{
  SqualeResult *result = NULL;
  zval *resource;
  
  if (zend_parse_parameters (ZEND_NUM_ARGS () TSRMLS_CC,
                             "r", &resource) == FAILURE) {
    return;
  }
  
  ZEND_FETCH_RESOURCE (result, SqualeResult *, &resource, -1,
                       le_squaleresult_name, le_squaleresult);
  
  if (result) {
    RETURN_LONG (result->status);
  }
  else {
    zend_error (E_WARNING, "Invalid result resource.");
    RETURN_NULL ();
  }
}
/* }}} */

/* {{{ proto long squale_result_get_processing_time (resource result)
   This function gives you the processing time for a result. That is basically
   the time in milliseconds that it took to run your query and gather data. */
PHP_FUNCTION (squale_result_get_processing_time)
{
  SqualeResult *result = NULL;
  zval *resource;
  
  if (zend_parse_parameters (ZEND_NUM_ARGS () TSRMLS_CC,
                             "r", &resource) == FAILURE) {
    return;
  }
  
  ZEND_FETCH_RESOURCE (result, SqualeResult *, &resource, -1,
                       le_squaleresult_name, le_squaleresult);
  
  if (result) {
    RETURN_LONG (result->processing_time);
  }
  else {
    zend_error (E_WARNING, "Invalid result resource.");
    RETURN_NULL ();
  }
}
/* }}} */

/* {{{ proto long squale_result_get_assignation_time (resource result)
   This function gives you the assignation time for a result. That is the delay
   in milliseconds before your job was actually really run by a worker after
   being put in the joblist as pending. */
PHP_FUNCTION (squale_result_get_assignation_time)
{
  SqualeResult *result = NULL;
  zval *resource;
  
  if (zend_parse_parameters (ZEND_NUM_ARGS () TSRMLS_CC,
                             "r", &resource) == FAILURE) {
    return;
  }
  
  ZEND_FETCH_RESOURCE (result, SqualeResult *, &resource, -1,
                       le_squaleresult_name, le_squaleresult);
  
  if (result) {
    RETURN_LONG (result->assignation_time);
  }
  else {
    zend_error (E_WARNING, "Invalid result resource.");
    RETURN_NULL ();
  }
}
/* }}} */

/* {{{ proto string squale_result_get_error (resource result)
   This function gives you the error message returned by SQuaLe. When no error
   it returns the empty string. */
PHP_FUNCTION (squale_result_get_error)
{
  SqualeResult *result = NULL;
  zval *resource;
  
  if (zend_parse_parameters (ZEND_NUM_ARGS () TSRMLS_CC,
                             "r", &resource) == FAILURE) {
    return;
  }
  
  ZEND_FETCH_RESOURCE (result, SqualeResult *, &resource, -1,
                       le_squaleresult_name, le_squaleresult);

  if ((!result) || (result->status != SQUALE_RESULT_ERROR)) {
    zend_error (E_WARNING, "Invalid result resource, or not an error.");
    RETURN_NULL ();
  }
  
  if (result->error) {
    RETURN_STRINGL (result->error, strlen (result->error), 1);
  }
  else {
    RETURN_EMPTY_STRING ();
  }
}
/* }}} */

/* {{{ proto string squale_result_get_warning (resource result)
   This function gives you the warning message returned by SQuaLe. When no 
   warning it returns the empty string. */
PHP_FUNCTION (squale_result_get_warning)
{
  SqualeResult *result = NULL;
  zval *resource;
  
  if (zend_parse_parameters (ZEND_NUM_ARGS () TSRMLS_CC,
                             "r", &resource) == FAILURE) {
    return;
  }
  
  ZEND_FETCH_RESOURCE (result, SqualeResult *, &resource, -1,
                       le_squaleresult_name, le_squaleresult);

  if ((!result) || (result->status != SQUALE_RESULT_RESULTSET_WITH_WARNING)) {
    zend_error (E_WARNING, "Invalid result resource, or not a resultset " \
                           "with warning.");
    RETURN_NULL ();
  }
  
  if (result->warning) {
    RETURN_STRINGL (result->warning, strlen (result->warning), 1);
  }
  else {
    RETURN_EMPTY_STRING ();
  }
}
/* }}} */

/* {{{ proto long squale_result_get_affected_rows (resource result)
   This function gives you the number of affected rows by your query. */
PHP_FUNCTION (squale_result_get_affected_rows)
{
  SqualeResult *result = NULL;
  zval *resource;
  
  if (zend_parse_parameters (ZEND_NUM_ARGS () TSRMLS_CC,
                             "r", &resource) == FAILURE) {
    return;
  }
  
  ZEND_FETCH_RESOURCE (result, SqualeResult *, &resource, -1,
                       le_squaleresult_name, le_squaleresult);
  
  if (result) {
    RETURN_LONG (result->affected_rows);
  }
  else {
    zend_error (E_WARNING, "Invalid result resource.");
    RETURN_NULL ();
  }
}
/* }}} */

/* {{{ proto long squale_result_get_nb_rows (resource result)
   This function returns the number of rows in the result. */
PHP_FUNCTION (squale_result_get_nb_rows)
{
  SqualeResult *result = NULL;
  zval *resource;
  
  if (zend_parse_parameters (ZEND_NUM_ARGS () TSRMLS_CC,
                             "r", &resource) == FAILURE) {
    return;
  }
  
  ZEND_FETCH_RESOURCE (result, SqualeResult *, &resource, -1,
                       le_squaleresult_name, le_squaleresult);
  
  if (result) {
    RETURN_LONG (result->num_rows);
  }
  else {
    zend_error (E_WARNING, "Invalid result resource.");
    RETURN_NULL ();
  }
}
/* }}} */

/* {{{ proto long squale_result_get_nb_cols (resource result)
   This function returns the number of rows in the result. */
PHP_FUNCTION (squale_result_get_nb_cols)
{
  SqualeResult *result = NULL;
  zval *resource;
  
  if (zend_parse_parameters (ZEND_NUM_ARGS () TSRMLS_CC,
                             "r", &resource) == FAILURE) {
    return;
  }
  
  ZEND_FETCH_RESOURCE (result, SqualeResult *, &resource, -1,
                       le_squaleresult_name, le_squaleresult);
  
  if (result) {
    RETURN_LONG (result->num_fields);
  }
  else {
    zend_error (E_WARNING, "Invalid result resource.");
    RETURN_NULL ();
  }
}
/* }}} */

/* {{{ proto array squale_result_get_column_names (resource result)
   This function returns an array of string containing the columns names. */
PHP_FUNCTION (squale_result_get_column_names)
{
  SqualeResult *result = NULL;
  int i;
  zval *resource;

  if (zend_parse_parameters (ZEND_NUM_ARGS () TSRMLS_CC,
                             "r", &resource) == FAILURE) {
    return;
  }
  
  ZEND_FETCH_RESOURCE (result, SqualeResult *, &resource, -1,
                       le_squaleresult_name, le_squaleresult);
  
  if ((!result) || (result->status != SQUALE_RESULT_RESULTSET) ||
      (!result->column_names)) {
    /* We don't have column names */
    zend_error (E_WARNING,
                "Invalid result resource, not a resultset, or no column names.");
    RETURN_NULL ();
  }
  
  array_init (return_value);
  
  for (i = 0; i < result->num_fields; i++) {
    add_index_stringl (return_value, i, result->column_names[i],
                       strlen (result->column_names[i]), 1);
  }
}
/* }}} */

/* {{{ proto string squale_result_get_value (resource result,
                                             long row, long column)
   This function returns a specific cell from the resultset. You specify the 
   coordinates that you want to get. */
PHP_FUNCTION (squale_result_get_value)
{
  SqualeResult *result = NULL;
  long row, col;
  zval *resource;

  if (zend_parse_parameters (ZEND_NUM_ARGS () TSRMLS_CC,
                             "rll", &resource, &row, &col) == FAILURE) {
    return;
  }
  
  ZEND_FETCH_RESOURCE (result, SqualeResult *, &resource, -1,
                       le_squaleresult_name, le_squaleresult);
  
  if ((!result) || (result->status != SQUALE_RESULT_RESULTSET) ||
      (!result->data)) {
    /* Not a resultset */
    zend_error (E_WARNING,
                "Invalid result resource, not a resultset, or no data.");
    RETURN_NULL ();
  }
  
  if ((row < 0) || (row > result->num_rows) ||
      (col < 0) || (col > result->num_fields)) {
    /* Error wrong coordinates */
    zend_error (E_WARNING, "Invalid row number for this result resource.");
    RETURN_NULL ();
  }
  
  if (result->data[row][col] == NULL) {
    RETURN_EMPTY_STRING ();
  }
  else {
    RETURN_STRINGL (result->data[row][col], strlen (result->data[row][col]), 1);
  }
}
/* }}} */

/* {{{ proto array squale_result_get_row (resource result, long row)
   This function returns an array of string for a row. */
PHP_FUNCTION (squale_result_get_row)
{
  SqualeResult *result = NULL;
  long row;
  int i;
  zval *resource;

  if (zend_parse_parameters (ZEND_NUM_ARGS () TSRMLS_CC,
                             "rl", &resource, &row) == FAILURE) {
    return;
  }
  
  ZEND_FETCH_RESOURCE (result, SqualeResult *, &resource, -1,
                       le_squaleresult_name, le_squaleresult);
  
  if ((!result) || (result->status != SQUALE_RESULT_RESULTSET) ||
      (!result->data)) {
    /* Not a resultset */
    zend_error (E_WARNING,
                "Invalid result resource, not a resultset, or no data.");
    RETURN_NULL ();
  }
  
  if ((row < 0) || (row > result->num_rows)) {
    /* Error wrong coordinates */
    zend_error (E_WARNING, "Invalid row number for this result resource.");
    RETURN_NULL ();
  }
  
  array_init (return_value);
  
  for (i = 0; i < result->num_fields; i++) {
    if (result->data[row][i] == NULL) {
      add_index_stringl (return_value, i, "", 0, 1);
    }
    else {
      add_index_stringl (return_value, i, result->data[row][i],
                         strlen (result->data[row][i]), 1);
    }
  }
}
/* }}} */

/* {{{ proto array squale_result_get_resultset (resource result)
   This function returns a 2 dimension array containing the whole resultset. */
PHP_FUNCTION (squale_result_get_resultset)
{
  SqualeResult *result = NULL;
  long i, j;
  zval *resource;

  if (zend_parse_parameters (ZEND_NUM_ARGS () TSRMLS_CC,
                             "r", &resource) == FAILURE) {
    return;
  }
  
  ZEND_FETCH_RESOURCE (result, SqualeResult *, &resource, -1,
                       le_squaleresult_name, le_squaleresult);
  
  if ((!result) || (result->status != SQUALE_RESULT_RESULTSET)) {
    /* Not a resultset */
    zend_error (E_WARNING, "Invalid result resource, or not a resultset.");
    RETURN_NULL ();
  }
  
  /* Empty resultset */
  if (!result->data) {
    RETURN_NULL ();
  }
  
  array_init (return_value);
  
  for (j = 0; j < result->num_rows; j++) {
    zval *row_array;
    MAKE_STD_ZVAL (row_array);
    array_init (row_array);
    for (i = 0; i < result->num_fields; i++) {
      if (result->data[j][i] == NULL) {
        add_index_stringl (row_array, i, "", 0, 1);
      }
      else {
        add_index_stringl (row_array, i, result->data[j][i],
                           strlen (result->data[j][i]), 1);
      }
    }
    add_index_zval (return_value, j, row_array);
  }
}
/* }}} */

/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
